#!/usr/bin/env perl
# Author: LrdShaper
# Date  : 02-may-2010
# perl script to extract program guide from Astro's new website
# and generate a xmltv.xml and ChannelInfo.xml
# Requires: Perl and the following perl modules
# Date::Calc
# XML::Writer
# File::Fetch
# File::Sort
###############################################################################

use Date::Calc qw(Add_Delta_Days Today);
use POSIX qw/strftime/;
use IO;
use XML::Writer;
use File::Sort qw(sort_file);
use File::Fetch;

$sourceFile = $ARGV[0];
$workDir = "./work";
$epgDir  = "./epgs";
$xmlDir  = "./xmls";
##--how many days of guide to fetch (not including today)?
$numDays = 7;

##--define the EPG url that we will be using
$epgURL = "http://www.astro.com.my/epg/epg.php?action=programme&cat=&genre=&classification=";

##--sort the ChannelInfo file
sort_file("${sourceFile}", "${workDir}/${sourceFile}.sorted");
##--make sure we can read the sorted ChannelInfo file
open(CHANNELS, "${workDir}/${sourceFile}.sorted") 
   or die "Cannot open input file: ${workDir}/${sourceFile}.sorted";

##--create the ChannelInfo.xml & xmltv.xml and write the root entity
my $channelOutput = new IO::File(">${xmlDir}/${sourceFile}.xml");
my $channelWriter = new XML::Writer( OUTPUT => $channelOutput, DATA_MODE => 'true', DATA_INDENT => 3 );
my $listingOutput = new IO::File(">${xmlDir}/xmltv.xml");
my $listingWriter = new XML::Writer( OUTPUT => $listingOutput, DATA_MODE => 'true', DATA_INDENT => 3 );
$channelWriter->xmlDecl('utf-8');
$channelWriter->startTag('NewDataSet');
$listingWriter->xmlDecl('utf-8');
$listingWriter->startTag( 'tv', 'generator-info-name' => 'AstroXMLTV', 'generator-info-url' => 'http://devrandom.blogsite.org/' );

while ( <CHANNELS> )
{
   if ((substr $_, 0, 1) eq '#')
   {
      next;
   }
   $_ =~ s/\r|\n//g;
   ##--split the fields separated by pipe
   ($channelID, $virtualchannel, $name, $channelURL) = split(/[|]/, $_);

   ##--create the ChannelInfo.xml entry
   $channelWriter->startTag('channel');
   $channelWriter->startTag('name');
   $channelWriter->characters($name);
   $channelWriter->endTag(  );
   $channelWriter->startTag('channelID');
   $channelWriter->characters($channelID);
   $channelWriter->endTag(  );
   $channelWriter->startTag('virtualchannel');
   $channelWriter->characters($virtualchannel);
   $channelWriter->endTag(  );
   $channelWriter->endTag(  );

   ##--create the xmltv.xml entry
   $listingWriter->startTag('channel', 'id' => "${channelID}");
   $listingWriter->startTag('display-name', 'lang' => 'en');
   $listingWriter->characters($name);
   $listingWriter->endTag(  );
   $listingWriter->startTag('url');
   $listingWriter->characters($channelURL);
   $listingWriter->endTag(  );
   $listingWriter->startTag('display-name', 'lang' => 'en');
   $listingWriter->characters($virtualchannel);
   $listingWriter->endTag(  );
   $listingWriter->startTag('url');
   $listingWriter->characters($channelURL);
   $listingWriter->endTag(  );
   $listingWriter->endTag(  );

   fetch the epg from Astro for this channel from today onwards
   ($dummy, $urlID) = split(/[=]/, $channelURL);
   for ($dateCtr = 0; $dateCtr <= $numDays; $dateCtr++)
   {
      $dateTS = sprintf("%d-%02d-%02d", Add_Delta_Days(Today(),$dateCtr));
      ##--build a File::Fetch object
      my $ff = File::Fetch->new(uri => "${epgURL}&channels=${urlID}&date=${dateTS}");
      ##--fetch the uri to ${epgDir}
      my $where = $ff->fetch( to => "${epgDir}" ) or die $ff->error;
      ##--rename the output file so we don't overwrite
      rename("${epgDir}/epg.php", "${epgDir}/${virtualchannel}.${dateCtr}.epg.php");
   }
}
##--close the ChannelInfo.xml
$channelWriter->endTag(  );
$channelWriter->end(  );

##--re-read the sorted ChannelInfo file, this time to read the EPGs
open(RECHANNELS, "${workDir}/${sourceFile}.sorted") 
   or die "Cannot open input file: ${workDir}/${sourceFile}.sorted";

while ( <RECHANNELS> )
{
   if ((substr $_, 0, 1) eq '#')
   {
      next;
   }
   $_ =~ s/\r|\n//g;
   ##--split the fields separated pipe
   ($channelID, $virtualchannel, $name, $channelURL) = split(/[|]/, $_);
   $initialShow = 1;
   for ($dateCtr = 0; $dateCtr <= $numDays; $dateCtr++)
   {
      open(SCHEDULE, "${epgDir}/${virtualchannel}.${dateCtr}.epg.php") 
         or die "Cannot open input file: ${epgDir}/${virtualchannel}.${dateCtr}.epg.php";
      while ( <SCHEDULE> )
      {
         ##--split the fields separated by title and put into an array
         @sourceFields = split /"title":"/;
         $arrayCtr = 1;
         while ( true )
         {
            @desiredLine  = split(/","/, $sourceFields[$arrayCtr]);
            if ($desiredLine[0] eq '')
            {
               last;
            }
            $title = $desiredLine[0];
            if (($title eq '(continuation)') || ($title eq '-'))
            {
               $arrayCtr++;
               next;
            }
            ($label, $episodetitle) = split(/":"/, $desiredLine[1]);
            if ($episodetitle eq '')
            {
               $episodetitle = none;
            }
            ($label, $datetime) = split(/":"/, "${desiredLine[2]}00");
            $datetime =~ s/-|:|\ //g;
            ($label, $duration) = split(/:/, $desiredLine[3]);
            ($duration, $label) = split(/}/, $duration);
            $arrayCtr++;
            ##--create the xmltv.xml entry
            if ($initialShow > 1)
            {
               $listingWriter->startTag('programme',
                                        'start' => "${prevDatetime} +0800",
                                        'stop' => "${datetime} +0800",
                                        'channel' => "${channelID}");
               $listingWriter->startTag('title', 'lang' => 'en');
               $listingWriter->characters($prevTitle);
               $listingWriter->endTag(  );
               $listingWriter->startTag('desc');
               $listingWriter->characters($prevEpisodetitle);
               $listingWriter->endTag(  );
               $listingWriter->endTag(  );
            }
            ($prevTitle, $prevEpisodetitle, $prevDatetime, $prevDuration) = ($title, $episodetitle, $datetime, $duration);
            $initialShow++;
         }
      }
   }
}
##--delete the sort file
unlink("${workDir}/${sourceFile}.sorted");

##--close the xmltv.xml
$listingWriter->endTag(  );
$listingWriter->end(  );

